#! /usr/bin/perl

use strict;
use POSIX qw(floor);

my $state = 0;
my $wal_level = '';
my $pages = 0;
my $binary = '';
my $scale = 0;
my $paramcount = 0;
my $mode = '';
my $sumtps = 0;
my $sumlat = 0;
my $count = 0;
my $trig = 0;
my $title = "(undef)";
my $ddltime = 0;
my @lines = ();
my $trailstr = '';
my $verbose = 0;
my $debug = 0;
my $summary = 0;
my %tps = ();
my %lat = ();
my %ddltime = ();
my %modestr=("n", "none", "s", "sync", "w", "WAL");
my $testset = 'undef';
my $datakey = 'undef';

foreach (@ARGV) {
	if ($_ eq '-v') {
		$verbose = 1;
	} elsif  ($_ eq '-s') {
		$summary = 1;
	} elsif  ($_ eq '-d') {
		$debug = 1;
	}
}

while (<STDIN>) {
	push(@lines, $_);
	chomp;
	next if (/^(END|START)$/);
	next if (/NOTICE:  /);

	print "$state: $_\n" if ($debug);
	if (/^## testset=(.*)$/) {
		$testset = $1;
	}
	if ($state == 0) {
		if (/^## params: count=([0-9.]+) pages=([0-9.]+) mode=(.) binary=([^ ]+) scale=([0-9.]+) wal_level=([a-z]+)/) {
			$paramcount = $1;
			$pages = $2;
			$mode = $3;
			$binary = $4;
			$scale = $5;
			$wal_level = $6;
			$datakey = "$testset:$pages:$mode";
			my $modestr = $modestr{$mode};

			print "## params: wal_level=$wal_level mode=$modestr pages=$pages count=$paramcount scale=$scale\n";
			print "(% are relative to \"before\")\n";
			$state = 1;
			next;
		} else {
			next;
		}
	} elsif ($state == 1) {
		if (/^starting vacuum/) {
			$state = 2;
		}
		next;
	} elsif ($state == 2) {
		if (/^bench.*/) {
			$trig = 1;
			$title = "before";
			$state = 3;
		}
	} elsif ($state == 3) {
		if (/^size ([0-9]+): procs ([0-9]+): time ([0-9]+)$/) {
			$ddltime{$datakey} = $3;
			$trig = 1;
			$title = "during";
			$trailstr = '';
			$state = 4;
		}
	} elsif ($state == 4) {
		if (/^transaction type: /) {
			$trig = 1;
			$title = "after";
			$trailstr = "# $_\n";
			$state = 5;
		}
	} elsif ($state == 5) {
		if (!/^statement latencies /) {
			$trailstr .= "# $_\n";
			next;
		}
		printf "DDL time: %6.0f ms (%6.1f%% relative to mode=%s)\n",
			$ddltime{$datakey},
			floor(1000.0 * $ddltime{$datakey} / $ddltime{"$testset:$pages:n"} + 0.5) / 10,
			$modestr{n};

		$trailstr .= "# $_\n";
		$state = 6;
		next;
	} elsif ($state == 6) {
		if (/^ {8}/) {
			$trailstr.= "# $_\n";
			next;
		}
		if ($verbose) {
			print $trailstr;
		}
		$state = 0;
		next;
	}

	if ($trig) {
		die "count 0?\n" if ($count == 0);
		my $thiskey = "$datakey:$title";
		my $beforekey = "$datakey:before";
		my $tps = $tps{$thiskey} = $sumtps / $count;
		my $lat = $lat{$thiskey} = $sumlat / $count;
		printf "%6s: tps %6.1f (%5.1f%%), lat %9.3f ms (%5.1f%%) (%d samples)\n",
			$title,
			$tps, floor(1000.0 * $tps / $tps{$beforekey} + 0.5)/10,
			$lat, floor(1000.0 * $lat / $lat{$beforekey} + 0.5)/10,
			$count;
		
		$sumtps = $sumlat = $count = 0;
		$trig = 0;
		next;
	}

	if (!/^progress: ([0-9.]+) s, ([0-9.]+) tps, lat ([0-9.]+) ms stddev ([0-9.]+|NaN)$/) {
		print "Unexpected line while state = $state at line $.\n";
		print "=====================================\n";
		foreach (-5 .. -1) {
			printf "%d: %s", ($. + $_ + 1), $lines[$. + $_];
		}
		print "=====================================\n";

		$state = 0;
	}
	$sumtps += $2;
	$sumlat += $3;
	$count++;
}


die "uncounted lines?\n" if ($count > 0);

if ($summary) {
	my %tmpts;
	print "=====================================\n";
	foreach (keys %ddltime) {
		my @t = split(/:/);
		$tmpts{$t[0]} = 1;
	}
	foreach my $testset (sort keys %tmpts) {
		print "Testset: $testset\n";
		print "\ttps-repl\ttps-sync\ttps-wal\tlat-repl\tlat-sync\tlat-wal\tDDLtime-repl\tDDLtime-sync\tDDLtime-wal\n";
		my %tmppages;
		foreach (keys %tps) {
			my @t = split(/:/);
			$tmppages{$t[1]} = 1;
		}
		foreach my $pages (sort {$a <=> $b } keys %tmppages) {
			my %reltps;
			my %rellat;

			# calculate perfromance drop by DDL workload
			foreach my $mode ('n', 's', 'w') {
				my $tpsb = $tps{"$testset:$pages:$mode:before"};
				my $tpsd = $tps{"$testset:$pages:$mode:during"};
				$reltps{$mode} = floor(1000 * $tpsd / $tpsb + 0.5) / 10;
				my $latb = $lat{"$testset:$pages:$mode:before"};
				my $latd = $lat{"$testset:$pages:$mode:during"};
				$rellat{$mode} = floor(1000 * $latd / $latb + 0.5) / 10;
			}

			my $ddln = $ddltime{"$testset:$pages:n"};
			my $ddls = $ddltime{"$testset:$pages:s"};
			my $ddlw = $ddltime{"$testset:$pages:w"};

			printf "%d\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\n",
				$pages,
				$reltps{n}, $reltps{s}, $reltps{w},
				$rellat{n}, $rellat{s}, $rellat{w},
				$ddln, $ddls, $ddlw;
		}
	}
}
